/**
 * Copyright (c) 2014-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.registry.cli.command;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.TreeSet;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.DefaultParser;
import org.apache.commons.cli.MissingArgumentException;
import org.apache.commons.cli.MissingOptionException;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.apache.commons.cli.UnrecognizedOptionException;
import org.ow2.petals.cli.api.command.AbstractCommand;
import org.ow2.petals.cli.api.command.CommandUtil;
import org.ow2.petals.cli.api.command.exception.CommandBadArgumentNumberException;
import org.ow2.petals.cli.api.command.exception.CommandException;
import org.ow2.petals.cli.api.command.exception.CommandInvalidException;
import org.ow2.petals.cli.api.command.exception.CommandMissingArgumentException;
import org.ow2.petals.cli.api.command.exception.CommandMissingOptionsException;

import com.ebmwebsourcing.easycommons.lang.CollectionHelper;
import com.ebmwebsourcing.easycommons.lang.CollectionHelper.Filter;

/**
 * This command list the topologies registered in the Petals Registry cluster.
 * 
 * @author Christophe DENEUX - Linagora
 */
public class ListTopology extends AbstractCommand {

    public static final String NAME = "list-topologies";

    protected static final String REGEX_SHORT_OPTION = "p";

    protected static final String REGEX_LONG_OPTION = "pattern";

    private static final Option REGEX_OPTION = Option.builder(REGEX_SHORT_OPTION).required(false)
            .longOpt(REGEX_LONG_OPTION)
            .desc(
                    "<topology-pattern> is an optional RegExp pattern to filter the content of the returned list. All the returned topologies will have an identifier matching the pattern.")
            .numberOfArgs(1).argName("topology-pattern").build();

    private final DefaultParser clParser;

    public ListTopology() {
        super(NAME);
        this.clParser = new DefaultParser();
        this.setUsage(CommandUtil.formatCommandUsage(this));
        this.setDescription("List the topologies registered");
        this.setOptionsDescription(CommandUtil.formatCommandOptionsDescription(this));
    }

    @Override
    public void doExecute(final String[] args) throws CommandException {
        try {
            final List<Filter<String>> filters = new ArrayList<Filter<String>>();
            final CommandLine cli = this.clParser.parse(getOptions(), args);
            if (cli.getArgs().length > 0) {
                throw new CommandBadArgumentNumberException(this);
            }

            if (cli.hasOption(REGEX_SHORT_OPTION)) {
                filters.add(new RegexFilter(cli.getOptionValue(REGEX_SHORT_OPTION)));
            }

            final Collection<String> topologies = Connect.PETALS_REGISTRY_CLIENT.getTopologies().keySet();

            final StringBuilder out = new StringBuilder();
            for (final String topologyId : this.sortAndFilterTopologies(topologies, filters)) {
                out.append(topologyId);
                out.append("\n");
            }
            if (!topologies.isEmpty()) {
                this.getShell().getPrintStream().print(out);
            }
        } catch (final UnrecognizedOptionException e) {
            throw new CommandBadArgumentNumberException(this, e);
        } catch (final MissingArgumentException e) {
            throw new CommandMissingArgumentException(this, e.getOption(), e);
        } catch (final MissingOptionException e) {
            throw new CommandMissingOptionsException(this, e.getMissingOptions(), e);
        } catch (final ParseException e) {
            throw new CommandInvalidException(this, e);
        }
    }

    /**
     * Sorts and filters topology identifiers
     * 
     * @param topologyIdentifiers
     *            The topology identifiers to sort and filter
     * @param filters
     *            The filters to apply
     * @return The topology identifiers sorted and filtered
     */
    private Collection<String> sortAndFilterTopologies(final Collection<String> topologyIdentifiers,
            final List<Filter<String>> filters) {
        final Collection<String> sortedTopologyIds = new TreeSet<String>(topologyIdentifiers);

        if (filters != null && !filters.isEmpty())
            CollectionHelper.filter(sortedTopologyIds, filters);

        return sortedTopologyIds;
    }

    @Override
    public Options createOptions() {
        final Options options = new Options();
        options.addOption(REGEX_OPTION);
        return options;
    }

    /**
     * Implements strict regex filter about topology
     * 
     * @author Christophe DENEUX - Linagora
     * 
     */
    private static class RegexFilter implements Filter<String> {
        private final Pattern pattern;

        public RegexFilter(final String regex) {
            this.pattern = Pattern.compile(regex);
        }

        @Override
        public boolean accept(final String topologyId) {
            final Matcher m = this.pattern.matcher(topologyId);
            return m.matches();
        }
    }

}
