/**
 * Copyright (c) 2014-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.registry.cli.command;

import static org.hamcrest.Matchers.containsString;
import static org.hamcrest.Matchers.endsWith;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Properties;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.DefaultParser;
import org.junit.Rule;
import org.junit.Test;
import org.ow2.petals.cli.api.command.exception.CommandBadArgumentNumberException;
import org.ow2.petals.cli.api.command.exception.CommandException;
import org.ow2.petals.cli.api.connection.ConnectionParameters;
import org.ow2.petals.cli.api.pref.Preferences;
import org.ow2.petals.cli.base.junit.shell.DummyShell;
import org.ow2.petals.cli.base.junit.shell.PetalsInteractiveCliUtils;
import org.ow2.petals.cli.pref.AbstractPreferencesImpl;
import org.ow2.petals.cli.shell.AbstractShell;
import org.ow2.petals.cli.shell.PetalsFileScriptShell;
import org.ow2.petals.registry.cli.Main;
import org.ow2.petals.registry.cli.connection.AuthenticatedConnectionParametersImpl;
import org.ow2.petals.registry.cli.pref.PreferencesImpl;
import org.ow2.petals.registry.client.mock.junit.PetalsRegistryClientApi;
import org.ow2.petals.registry_overlay.api.Constants;
/**
 * Unit test of {@link Connect}
 *
 * @author Christophe DENEUX - Linagora
 *
 */
public class ConnectTest extends AbstractArtifactCommandTest {

    private final static String ALIAS_IN_PREF_FILE = "test-alias";

    private final static String HOST_IN_PREF_FILE = "localhost";

    private final static String PORT_IN_PREF_FILE = String.valueOf(Constants.DEFAULT_PORT);

    private final static String GROUP_IN_PREF_FILE = "petals";

    private final static String PASSWORD_IN_PREF_FILE = "petals";

    /**
     * We use the mock implementation of Petals Registry Client API
     */
    @Rule
    public final PetalsRegistryClientApi prca = new PetalsRegistryClientApi();

    private void setUpEnvFile() throws Exception {
        setUpEnvFile(false, false);
    }

    private void setUpEnvFile(final boolean group, final boolean password) throws Exception {

        final Properties properties = new Properties();
        final String alias = ALIAS_IN_PREF_FILE;
        properties.setProperty(AbstractPreferencesImpl.DEFAULT_ALIAS_PROPERTY, alias);
        properties.setProperty(alias + AbstractPreferencesImpl.HOST_PROPERTY_SUFFIX, HOST_IN_PREF_FILE);
        properties.setProperty(alias + AbstractPreferencesImpl.PORT_PROPERTY_SUFFIX, PORT_IN_PREF_FILE);

        if (group) {
            properties.setProperty(alias + PreferencesImpl.GROUP_PROPERTY_SUFFIX, GROUP_IN_PREF_FILE);
        }

        if (password) {
            properties.setProperty(alias + PreferencesImpl.PASSWORD_PROPERTY_SUFFIX, PASSWORD_IN_PREF_FILE);
        }

        setUpPrefFile(properties);
    }

    /**
     * Test that the command usage is defined
     */
    @Test
    public void testUsage_0() {

        final DummyShell dummyShell = new DummyShell();
        final Connect command = new Connect();
        command.setShell(dummyShell);
        final String usage = command.getUsage();
        assertNotNull("Command usage null", usage);
        assertFalse("Command usage empty", usage.isEmpty());
        assertTrue("'usage' is missing at the begining of the command usage",
                usage.startsWith("usage"));
        assertTrue("The command name is missing in the command usage",
                usage.contains(command.getName()));
    }

    /**
     * Test of the command "connect" against argument management, where:
     * <ul>
     * <li>an unregonized option is provided</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void testArgumentsError_00() throws CommandException {
        final DummyShell dummyShell = new DummyShell();
        final Connect command = new Connect();
        command.setShell(dummyShell);
        try {
            command.execute(new String[] { "-z" });
            fail("CommandBadArgumentNumberException is not thrown");
        } catch (final CommandBadArgumentNumberException e) {
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    CommandBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test of the command "connect" against argument management, where:
     * <ul>
     * <li>using an alias without groupname/password,</li>
     * <li>without yes flag,</li>
     * <li>using an interactive shell.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>groupname and password are asked,</li>
     * <li>after to have entered groupname and password, the command is executed displaying the prompt</li>
     * </ul>
     * </p>
     */
    @Test
    public void testArgumentsError_01() throws Exception {

        setUpEnvFile();

        try {
            // Enable the hack for Windows?
            PetalsInteractiveCliUtils.hackForWindows(true);

            this.runCli("-C");

            // Wait that Petals CLI starts
            checkOut(containsString(Main.PROMPT_BASE + ">"));

            // Enter the command
            systemIn.writeLine("connect -a " + ALIAS_IN_PREF_FILE);

            // Wait the execution of the command
            checkOut(endsWith(Connect.GROUP_QUESTION));

            // Enter groupname
            systemIn.writeLine("petals");

            // Wait the execution of the command
            checkOut(endsWith(Connect.PWD_QUESTION));

            // Enter password
            systemIn.writeLine("petals");

            // Wait the execution of the command
            checkOut(endsWith(Main.PROMPT_BASE + "@" + HOST_IN_PREF_FILE + ":" + PORT_IN_PREF_FILE + ">"));

            systemIn.writeLine("exit");

            waitThreadOrFail();
        } finally {
            // Disable the hack for Windows
            PetalsInteractiveCliUtils.hackForWindows(false);
        }
    }

    /**
     * Test of the command "connect" against argument management, where:
     * <ul>
     * <li>using an alias without groupname/password,</li>
     * <li>with yes flag as command option,</li>
     * <li>using an interactive shell.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>groupname and password are not asked,</li>
     * <li>an error occurs about missing groupname and password</li>
     * </ul>
     * </p>
     */
    @Test
    public void testArgumentsError_02() throws Exception {

        setUpEnvFile();

        try {
            // Enable the hack for Windows?
            PetalsInteractiveCliUtils.hackForWindows(true);

            runCli("-C");

            // Wait that Petals CLI starts
            checkOut(containsString(Main.PROMPT_BASE + ">"));

            // Enter the command
            systemIn.writeLine("connect -a " + ALIAS_IN_PREF_FILE + " -y");

            // Wait the execution of the command
            checkErrEndsWith(Connect.MISSING_GROUP_AND_PWD);
            checkOut(containsString(Main.PROMPT_BASE + ">"));

            systemIn.writeLine("exit");

            waitThreadOrFail();
        } finally {
            // Disable the hack for Windows
            PetalsInteractiveCliUtils.hackForWindows(false);
        }
    }

    /**
     * Test of the command "connect" against argument management, where:
     * <ul>
     * <li>using an alias without groupname/password,</li>
     * <li>with yes flag on the command line,</li>
     * <li>using an interactive shell.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>groupname and password are not asked,</li>
     * <li>an error occurs about missing groupname and password</li>
     * </ul>
     * </p>
     */
    @Test
    public void testArgumentsError_03() throws Exception {

        setUpEnvFile();

        try {
            // Enable the hack for Windows?
            PetalsInteractiveCliUtils.hackForWindows(true);

            runCli("-y", "-C");

            // Wait that Petals CLI starts
            checkOut(containsString(Main.PROMPT_BASE + ">"));

            // Enter the command
            systemIn.writeLine("connect -a " + ALIAS_IN_PREF_FILE);

            // Wait the execution of the command
            checkErrEndsWith(Connect.MISSING_GROUP_AND_PWD);
            checkOut(containsString(Main.PROMPT_BASE + ">"));

            systemIn.writeLine("exit");

            waitThreadOrFail();

        } finally {
            // Disable the hack for Windows
            PetalsInteractiveCliUtils.hackForWindows(false);
        }
    }

    /**
     * Test of the command "connect" against argument management, where:
     * <ul>
     * <li>using an alias with a groupname and without password,</li>
     * <li>without yes flag,</li>
     * <li>using an interactive shell.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>an error occurs about the missing password and it is not asked</li>
     * <li>the interactive shell ends,</li>
     * <li>the exit code is correctly set to 1.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testArgumentsError_07() throws Exception {

        setUpEnvFile(true, false);

        try {
            // Enable the hack for Windows?
            PetalsInteractiveCliUtils.hackForWindows(true);

            runCli(true, "-C");

            // Wait that Petals CLI starts
            checkErr(containsString(ALIAS_IN_PREF_FILE));
            assertTrue("Something is printed on output", systemOut.getLog().isEmpty());

            systemIn.writeLine("exit");

            waitThreadOrFail();
        } finally {
            // Disable the hack for Windows
            PetalsInteractiveCliUtils.hackForWindows(false);
        }
    }

    /**
     * Test of the command "connect" against argument management, where:
     * <ul>
     * <li>using an alias without a groupname and with a password,</li>
     * <li>without yes flag,</li>
     * <li>using an interactive shell.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>an error occurs about the missing groupname and it is not asked</li>
     * <li>the interactive shell ends,</li>
     * <li>the exit code is correctly set to 1.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testArgumentsError_08() throws Exception {

        setUpEnvFile(false, true);

        try {
            // Enable the hack for Windows?
            PetalsInteractiveCliUtils.hackForWindows(true);

            runCli(true, "-C");

            // Wait that Petals CLI starts
            checkErr(containsString(ALIAS_IN_PREF_FILE));
            assertTrue("Something is printed on output", systemOut.getLog().isEmpty());

            systemIn.writeLine("exit");

            waitThreadOrFail();
        } finally {
            // Disable the hack for Windows
            PetalsInteractiveCliUtils.hackForWindows(false);
        }
    }

    /**
     * Test of the command "connect" against argument management, where:
     * <ul>
     * <li>using an alias without groupname/password,</li>
     * <li>without yes flag,</li>
     * <li>using a non-interactive shell.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about missing groupname and password.
     * </p>
     */
    @Test
    public void testArgumentsError_04() throws Exception {

        setUpEnvFile();

        final Connect command = new Connect();

        // Create the Petals CLI script
        final File petalsCliScript = this.tempFolder.newFile("petals-registry-cli.cmd");
        final OutputStream osScript = new FileOutputStream(petalsCliScript);
        final Writer pwScript = new OutputStreamWriter(osScript);
        try {
            pwScript.write(command.getName());
            pwScript.write(" -");
            pwScript.write(Connect.ALIAS_SHORT_OPTION);
            pwScript.write(ALIAS_IN_PREF_FILE);
            pwScript.write('\n');
        } finally {
            pwScript.close();
            osScript.close();
        }

        final Preferences pref = new PreferencesImpl();
        pref.intializePreferenceParameters();
        final PetalsFileScriptShell shell = new PetalsFileScriptShell(petalsCliScript.getAbsolutePath(), null, false,
                false, pref, null, null);
        shell.registersCommand(command);
        command.setShell(shell);

        shell.run();

        final String error = systemErr.getLog();
        assertFalse("No error message print", error.isEmpty());
        assertTrue("It's not the expected error message.", error.contains(Connect.MISSING_GROUP_AND_PWD));

    }

    /**
     * Test of the command "connect" against argument management, where:
     * <ul>
     * <li>using an alias without groupname/password,</li>
     * <li>with the yes flag set as command option,</li>
     * <li>using a non-interactive shell.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about missing groupname and password.
     * </p>
     */
    @Test
    public void testArgumentsError_05() throws Exception {

        setUpEnvFile();

        final Connect command = new Connect();

        // Create the Petals CLI script
        final File petalsCliScript = this.tempFolder.newFile("petals-registry-cli.cmd");
        final OutputStream osScript = new FileOutputStream(petalsCliScript);
        final Writer pwScript = new OutputStreamWriter(osScript);
        try {
            pwScript.write(command.getName());
            pwScript.write(" -");
            pwScript.write(Connect.ALIAS_SHORT_OPTION);
            pwScript.write(ALIAS_IN_PREF_FILE);
            pwScript.write(" -");
            pwScript.write(Connect.YESFLAG_SHORT_OPTION);
            pwScript.write('\n');
        } finally {
            pwScript.close();
            osScript.close();
        }

        final Preferences pref = new PreferencesImpl();
        pref.intializePreferenceParameters();
        final PetalsFileScriptShell shell = new PetalsFileScriptShell(petalsCliScript.getAbsolutePath(), null, false,
                false, pref, null, null);
        shell.registersCommand(command);
        command.setShell(shell);

        shell.run();

        final String error = systemErr.getLog();
        assertFalse("No error message print", error.isEmpty());
        assertTrue("It's not the expected error message.", error.contains(Connect.MISSING_GROUP_AND_PWD));

    }

    /**
     * Test of the command "connect" against argument management, where:
     * <ul>
     * <li>using an alias without groupname/password,</li>
     * <li>with the yes flag set on the command line,</li>
     * <li>using a non-interactive shell.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about missing groupname and password.
     * </p>
     */
    @Test
    public void testArgumentsError_06() throws Exception {


        setUpEnvFile();

        final Connect command = new Connect();

        // Create the Petals CLI script
        final File petalsCliScript = this.tempFolder.newFile("petals-registry-cli.cmd");
        final OutputStream osScript = new FileOutputStream(petalsCliScript);
        final Writer pwScript = new OutputStreamWriter(osScript);
        try {
            pwScript.write(command.getName());
            pwScript.write(" -");
            pwScript.write(Connect.ALIAS_SHORT_OPTION);
            pwScript.write(ALIAS_IN_PREF_FILE);
            pwScript.write('\n');
        } finally {
            pwScript.close();
            osScript.close();
        }

        final Preferences pref = new PreferencesImpl();
        pref.intializePreferenceParameters();
        final PetalsFileScriptShell shell = new PetalsFileScriptShell(petalsCliScript.getAbsolutePath(), null, false,
                true, pref, null, null);
        shell.registersCommand(command);
        command.setShell(shell);

        shell.run();

        final String error = systemErr.getLog();
        assertFalse("No error message print", error.isEmpty());
        assertTrue("It's not the expected error message.", error.contains(Connect.MISSING_GROUP_AND_PWD));

    }

    /**
     * Test of the command "connect" against argument management, where:
     * <ul>
     * <li>using an alias with groupname/password,</li>
     * <li>without yes flag,</li>
     * <li>using a non-interactive shell.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: no error occurs.
     * </p>
     */
    @Test
    public void testArguments_00() throws Exception {

        setUpEnvFile(true, true);

        final Connect command = new Connect();

        // Create the Petals CLI script
        final File petalsCliScript = this.tempFolder.newFile("petals-registry-cli.cmd");
        final OutputStream osScript = new FileOutputStream(petalsCliScript);
        final Writer pwScript = new OutputStreamWriter(osScript);
        try {
            pwScript.write(command.getName());
            pwScript.write(" -");
            pwScript.write(Connect.ALIAS_SHORT_OPTION);
            pwScript.write(ALIAS_IN_PREF_FILE);
            pwScript.write('\n');
        } finally {
            pwScript.close();
            osScript.close();
        }

        final Preferences pref = new PreferencesImpl();
        pref.intializePreferenceParameters();
        final PetalsFileScriptShell shell = new PetalsFileScriptShell(petalsCliScript.getAbsolutePath(), null, false,
                false, pref, null, null);
        shell.registersCommand(command);
        command.setShell(shell);

        shell.run();

        final String error = systemErr.getLog();
        assertTrue("An error message is printed", error.isEmpty());
        assertEquals("Unexpected exit code", 0, shell.getExitStatus());

    }

    /**
     * Test of the command "connect" against argument management, where:
     * <ul>
     * <li>using an alias with groupname/password,</li>
     * <li>with a yes flag set as command option,</li>
     * <li>using a non-interactive shell.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: no error occurs.
     * </p>
     */
    @Test
    public void testArguments_01() throws Exception {

        setUpEnvFile(true, true);

        final Connect command = new Connect();

        // Create the Petals CLI script
        final File petalsCliScript = this.tempFolder.newFile("petals-registry-cli.cmd");
        final OutputStream osScript = new FileOutputStream(petalsCliScript);
        final Writer pwScript = new OutputStreamWriter(osScript);
        try {
            pwScript.write(command.getName());
            pwScript.write(" -");
            pwScript.write(Connect.ALIAS_SHORT_OPTION);
            pwScript.write(ALIAS_IN_PREF_FILE);
            pwScript.write(" -");
            pwScript.write(Connect.YESFLAG_SHORT_OPTION);
            pwScript.write('\n');
        } finally {
            pwScript.close();
            osScript.close();
        }

        final Preferences pref = new PreferencesImpl();
        pref.intializePreferenceParameters();
        final PetalsFileScriptShell shell = new PetalsFileScriptShell(petalsCliScript.getAbsolutePath(), null, false,
                false, pref, null, null);
        shell.registersCommand(command);
        command.setShell(shell);

        shell.run();

        final String error = systemErr.getLog();
        assertTrue("An error message is printed", error.isEmpty());
        assertEquals("Unexpected exit code", 0, shell.getExitStatus());

    }

    /**
     * Test of the command "connect" against argument management, where:
     * <ul>
     * <li>using an alias with groupname/password,</li>
     * <li>with the yes flag set on the command line,</li>
     * <li>using a non-interactive shell.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: no error occurs.
     * </p>
     */
    @Test
    public void testArguments_02() throws Exception {

        setUpEnvFile(true, true);

        final Connect command = new Connect();

        // Create the Petals CLI script
        final File petalsCliScript = this.tempFolder.newFile("petals-registry-cli.cmd");
        final OutputStream osScript = new FileOutputStream(petalsCliScript);
        final Writer pwScript = new OutputStreamWriter(osScript);
        try {
            pwScript.write(command.getName());
            pwScript.write(" -");
            pwScript.write(Connect.ALIAS_SHORT_OPTION);
            pwScript.write(ALIAS_IN_PREF_FILE);
            pwScript.write('\n');
        } finally {
            pwScript.close();
            osScript.close();
        }

        final Preferences pref = new PreferencesImpl();
        pref.intializePreferenceParameters();
        final PetalsFileScriptShell shell = new PetalsFileScriptShell(petalsCliScript.getAbsolutePath(), null, false,
                true, pref, null, null);
        shell.registersCommand(command);
        command.setShell(shell);

        shell.run();

        final String error = systemErr.getLog();
        assertTrue("An error message is printed", error.isEmpty());
        assertEquals("Unexpected exit code", 0, shell.getExitStatus());

    }

    private static final String DEFAULT_HOST = "1.1.1.1";

    private static final int DEFAULT_PORT = 4875;

    private static final String DEFAULT_ALIAS = "default";

    private static final String ALIAS_1 = "alias-1";

    private static final String ALIAS_1_HOST = "localhost";

    private static final int ALIAS_1_PORT = 7845;

    private static final String ALIAS_3 = "alias-3";

    private static final String ALIAS_3_HOST = "2.2.2.2";

    private static final int ALIAS_3_PORT = 4123;

    private static final String ALIAS_3_GROUOP = "testGroup";

    private static final String ALIAS_3_PWD = "testPassword";

    private File createPreferenceFile() throws IOException {
        final File prefFile = this.tempFolder.newFile();
        final Properties preferences = new Properties();
        preferences.setProperty(AbstractPreferencesImpl.DEFAULT_ALIAS_PROPERTY, "default");
        preferences.setProperty(DEFAULT_ALIAS + AbstractPreferencesImpl.HOST_PROPERTY_SUFFIX, DEFAULT_HOST);
        preferences.setProperty(DEFAULT_ALIAS + AbstractPreferencesImpl.PORT_PROPERTY_SUFFIX,
                String.valueOf(DEFAULT_PORT));
        preferences.setProperty(ALIAS_1 + AbstractPreferencesImpl.HOST_PROPERTY_SUFFIX, ALIAS_1_HOST);
        preferences.setProperty(ALIAS_1 + AbstractPreferencesImpl.PORT_PROPERTY_SUFFIX, String.valueOf(ALIAS_1_PORT));
        preferences.setProperty(ALIAS_3 + AbstractPreferencesImpl.HOST_PROPERTY_SUFFIX, ALIAS_3_HOST);
        preferences.setProperty(ALIAS_3 + AbstractPreferencesImpl.PORT_PROPERTY_SUFFIX, String.valueOf(ALIAS_3_PORT));
        preferences.setProperty(ALIAS_3 + PreferencesImpl.GROUP_PROPERTY_SUFFIX, ALIAS_3_GROUOP);
        preferences.setProperty(ALIAS_3 + PreferencesImpl.PASSWORD_PROPERTY_SUFFIX, ALIAS_3_PWD);
        preferences.store(new FileOutputStream(prefFile), "");
        return prefFile;
    }

    /**
     * A classloader reading any file as preference file
     */
    public class PrefFileClassLoader extends ClassLoader {

        private final File prefFile;

        public PrefFileClassLoader(final File prefFile) {
            this.prefFile = prefFile;
        }

        @Override
        public URL getResource(final String name) {
            if (PreferencesImpl.PREFERENCE_FILE_NAME.equals(name)) {
                try {
                    return this.prefFile.toURI().toURL();
                } catch (final MalformedURLException e) {
                    // Should not occur
                    return null;
                }
            } else {
                return super.getResource(name);
            }
        }
    }

    @Test
    public final void testParseConnectionParametersWithNothingAndUseDefault() throws Exception {

        final File prefFile = this.createPreferenceFile();

        final ClassLoader oldClassLoader = Thread.currentThread().getContextClassLoader();
        Thread.currentThread().setContextClassLoader(new PrefFileClassLoader(prefFile));
        try {

            final Connect command = new Connect();
            final Preferences pref = new PreferencesImpl();
            pref.intializePreferenceParameters();
            command.setShell(new AbstractShell(System.out, System.err, false, false, pref, null, "") {
                @Override
                public void run() {
                    // NOP
                }
            });
            final CommandLineParser parser = new DefaultParser();
            final CommandLine cmd = parser.parse(command.getOptions(), new String[] { command.getName() });

            final ConnectionParameters connectionParameters = command.parseConnectionParameters(cmd, true);
            assertNotNull(connectionParameters);
            assertEquals(new ConnectionParameters(DEFAULT_HOST, DEFAULT_PORT), connectionParameters);
        } finally {
            Thread.currentThread().setContextClassLoader(oldClassLoader);
        }
    }

    @Test
    public final void testParseConnectionParametersWithNothingNotUseDefault() throws Exception {

        final File prefFile = this.createPreferenceFile();

        final ClassLoader oldClassLoader = Thread.currentThread().getContextClassLoader();
        Thread.currentThread().setContextClassLoader(new PrefFileClassLoader(prefFile));
        try {

            final Connect command = new Connect();
            command.setShell(new AbstractShell(System.out, System.err, false, false, new PreferencesImpl(), null, "") {
                @Override
                public void run() {
                    // NOP
                }
            });
            final CommandLineParser parser = new DefaultParser();
            final CommandLine cmd = parser.parse(command.getOptions(), new String[] { command.getName() });

            final ConnectionParameters connectionParameters = command.parseConnectionParameters(cmd, false);
            assertNull(connectionParameters);
        } finally {
            Thread.currentThread().setContextClassLoader(oldClassLoader);
        }
    }

    @Test
    public final void testParseConnectionParametersWithAlias() throws Exception {

        final File prefFile = this.createPreferenceFile();

        final ClassLoader oldClassLoader = Thread.currentThread().getContextClassLoader();
        Thread.currentThread().setContextClassLoader(new PrefFileClassLoader(prefFile));
        try {

            final Connect command = new Connect();
            final Preferences pref = new PreferencesImpl();
            pref.intializePreferenceParameters();
            command.setShell(new AbstractShell(System.out, System.err, false, false, pref, null, "") {
                @Override
                public void run() {
                    // NOP
                }
            });
            final CommandLineParser parser = new DefaultParser();
            final CommandLine cmd = parser.parse(command.getOptions(), new String[] { command.getName(),
                    "-" + Connect.ALIAS_SHORT_OPTION, ALIAS_3 });

            final ConnectionParameters connectionParameters = command.parseConnectionParameters(cmd, true);
            assertNotNull(connectionParameters);
            assertEquals(new AuthenticatedConnectionParametersImpl(ALIAS_3_HOST, ALIAS_3_PORT, ALIAS_3_GROUOP,
                    ALIAS_3_PWD), connectionParameters);
        } finally {
            Thread.currentThread().setContextClassLoader(oldClassLoader);
        }
    }

    @Test
    public final void testParseConnectionParametersWithParameters() throws Exception {

        final File prefFile = this.createPreferenceFile();

        final ClassLoader oldClassLoader = Thread.currentThread().getContextClassLoader();
        Thread.currentThread().setContextClassLoader(new PrefFileClassLoader(prefFile));
        try {
            final String expectedHost = "localhost";
            final int expectedPort = Constants.DEFAULT_PORT;
            final String expectedGroup = "group";
            final String expectedPwd = "pwd";

            final Connect command = new Connect();
            command.setShell(new AbstractShell(System.out, System.err, false, false, new PreferencesImpl(), null, "") {
                @Override
                public void run() {
                    // NOP
                }
            });
            final CommandLineParser parser = new DefaultParser();
            final CommandLine cmd = parser.parse(command.getOptions(),
                    new String[] { command.getName(), "-" + Connect.HOST_SHORT_OPTION, expectedHost,
                            "-" + Connect.PORT_SHORT_OPTION, String.valueOf(expectedPort),
                            "-" + Connect.GROUP_SHORT_OPTION, expectedGroup, "-" + Connect.PWD_SHORT_OPTION,
                            expectedPwd });

            final ConnectionParameters connectionParameters = command.parseConnectionParameters(cmd, true);
            assertNotNull(connectionParameters);
            assertEquals(new AuthenticatedConnectionParametersImpl(expectedHost, expectedPort, expectedGroup,
                    expectedPwd), connectionParameters);
        } finally {
            Thread.currentThread().setContextClassLoader(oldClassLoader);
        }
    }

    @Test(expected = CommandException.class)
    public final void testParseConnectionParametersWithIncompatibleArguments() throws Exception {

        final File prefFile = this.createPreferenceFile();

        final ClassLoader oldClassLoader = Thread.currentThread().getContextClassLoader();
        Thread.currentThread().setContextClassLoader(new PrefFileClassLoader(prefFile));
        try {
            final String expectedHost = "localhost";
            final int expectedPort = Constants.DEFAULT_PORT;
            final String expectedGroup = "group";
            final String expectedPwd = "pwd";

            final Connect command = new Connect();
            command.setShell(new AbstractShell(System.out, System.err, false, false, new PreferencesImpl(), null, "") {
                @Override
                public void run() {
                    // NOP
                }
            });
            final CommandLineParser parser = new DefaultParser();
            final CommandLine cmd = parser.parse(command.getOptions(),
                    new String[] { command.getName(), "-" + Connect.HOST_SHORT_OPTION, expectedHost,
                            "-" + Connect.PORT_SHORT_OPTION, String.valueOf(expectedPort),
                            "-" + Connect.GROUP_SHORT_OPTION, expectedGroup, "-" + Connect.PWD_SHORT_OPTION,
                            expectedPwd, "-" + Connect.ALIAS_SHORT_OPTION, ALIAS_1 });

            try {
                command.parseConnectionParameters(cmd, true);
            } catch (final CommandException ce) {
                assertEquals(CommandBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS,
                        ce.getMessage());
                throw ce;
            }
        } finally {
            Thread.currentThread().setContextClassLoader(oldClassLoader);
        }
    }

    @Test(expected = CommandException.class)
    public final void testParseConnectionParametersWithBadNumberOfArguments() throws Exception {

        final File prefFile = this.createPreferenceFile();

        final ClassLoader oldClassLoader = Thread.currentThread().getContextClassLoader();
        Thread.currentThread().setContextClassLoader(new PrefFileClassLoader(prefFile));
        try {
            final String expectedHost = "localhost";
            final int expectedPort = Constants.DEFAULT_PORT;

            final Connect command = new Connect();
            command.setShell(new AbstractShell(System.out, System.err, false, false, new PreferencesImpl(), null, "") {
                @Override
                public void run() {
                    // NOP
                }
            });
            final CommandLineParser parser = new DefaultParser();
            final CommandLine cmd = parser.parse(command.getOptions(),
                    new String[] { command.getName(), "-" + Connect.HOST_SHORT_OPTION, expectedHost,
                            "-" + Connect.PORT_SHORT_OPTION, String.valueOf(expectedPort) });

            try {
                command.parseConnectionParameters(cmd, true);
            } catch (final CommandException ce) {
                assertEquals(CommandBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS,
                        ce.getMessage());
                throw ce;
            }
        } finally {
            Thread.currentThread().setContextClassLoader(oldClassLoader);
        }
    }
}
