/**
 * Copyright (c) 2015-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.registry.cli.command;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;
import static org.ow2.petals.cli.base.junit.Assert.assertCommandMissingOptionsException;

import org.junit.Rule;
import org.junit.Test;
import org.ow2.petals.cli.api.command.Command;
import org.ow2.petals.cli.api.command.exception.CommandBadArgumentNumberException;
import org.ow2.petals.cli.api.command.exception.CommandException;
import org.ow2.petals.cli.api.command.exception.CommandMissingArgumentException;
import org.ow2.petals.cli.api.command.exception.CommandMissingOptionsException;
import org.ow2.petals.cli.api.command.exception.ConnectionRequiredException;
import org.ow2.petals.cli.api.shell.Shell;
import org.ow2.petals.cli.base.junit.shell.DummyShell;
import org.ow2.petals.cli.base.junit.shell.StringStreamShell;
import org.ow2.petals.registry.client.api.exception.ConnectionErrorException;
import org.ow2.petals.registry.client.api.exception.UnexistingTopologyException;
import org.ow2.petals.registry.client.mock.junit.PetalsRegistryClientApi;
import org.ow2.petals.registry_overlay.api.Constants;
import org.ow2.petals.topology.generated.Topology;

/**
 * Unit tests of the command {@link DeleteTopology}
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class DeleteTopologyTest {

    @Rule
    public final PetalsRegistryClientApi prca = new PetalsRegistryClientApi();

    /**
     * Test that the command usage is defined
     */
    @Test
    public void usage() {

        final Shell shell = new DummyShell();
        final Command command = new DeleteTopology();
        command.setShell(shell);
        final String usage = command.getUsage();
        assertNotNull("Command usage null", usage);
        assertFalse("Command usage empty", usage.isEmpty());
        assertTrue("'usage' is missing at the begining of the command usage", usage.startsWith("usage"));
        assertTrue("The command name is missing in the command usage", usage.contains(command.getName()));
    }

    /**
     * <p>
     * Test of the command {@value DeleteTopology#NAME} to delete a topology of the Petals Registry when no topology was
     * previsously registered.
     * </p>
     * <p>
     * Expected results: an error occurs about the unexisting topology
     * </p>
     */
    @Test
    public void executeWithoutTopologyRegistered() throws CommandException, ConnectionErrorException {
        final StringStreamShell shell = new StringStreamShell();

        Connect.PETALS_REGISTRY_CLIENT = this.prca.getPetalsRegistryClientFactory().createClient("localhost",
                Constants.DEFAULT_PORT, Constants.DEFAULT_CREDENTIALS_GROUP, Constants.DEFAULT_CREDENTIALS_PASSWORD);
        shell.onConnectionEstablished();

        final Command command = new DeleteTopology();
        try {
            command.setShell(shell);
            command.execute(new String[] { "-" + DeleteTopology.TOPOLOGY_SHORT_OPTION, "unexisting-topology" });
            fail("CommandException is not thrown");
        } catch (final CommandException e) {
            if (e.getCause() instanceof UnexistingTopologyException) {
                // Expected exception
                assertEquals(command, e.getCommand());
            } else {
                fail("Unexpected cause");
            }
        } finally {
            Connect.PETALS_REGISTRY_CLIENT = null;
        }
    }

    /**
     * <p>
     * Test of the command {@value DeleteTopology#NAME} to delete a topology of the Petals Registry when several
     * topologies are previously registered.
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error occurs</li>
     * <li>the expected topology is correctly deleted</li>
     * </ul>
     * </p>
     */
    @Test
    public void executeWithSeveralTopologiesRegistered() throws CommandException, ConnectionErrorException {

        final String topologyToDelete = "topology";
        final String topology_1 = topologyToDelete;
        final String topology_2 = "topology2";
        this.prca.registerTopology(topology_2, new Topology());
        this.prca.registerTopology(topology_1, new Topology());
        assertEquals(2, this.prca.getRegisteredTopology().size());

        final StringStreamShell shell = new StringStreamShell();

        Connect.PETALS_REGISTRY_CLIENT = this.prca.getPetalsRegistryClientFactory().createClient("localhost",
                Constants.DEFAULT_PORT, Constants.DEFAULT_CREDENTIALS_GROUP, Constants.DEFAULT_CREDENTIALS_PASSWORD);
        shell.onConnectionEstablished();
        try {
            final Command command = new DeleteTopology();
            command.setShell(shell);
            command.execute(new String[] { "-" + DeleteTopology.TOPOLOGY_SHORT_OPTION, topologyToDelete });

            assertEquals(1, this.prca.getRegisteredTopology().size());
            assertTrue(this.prca.getRegisteredTopology().containsKey(topology_2));

        } finally {
            Connect.PETALS_REGISTRY_CLIENT = null;
        }
    }

    /**
     * Test of the command {@value DeleteTopology#NAME} against argument management, where:
     * <ul>
     * <li>an unregonized option is provided</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void errorUnrecognizedOption() throws CommandException, ConnectionErrorException {

        final Shell shell = new StringStreamShell();

        Connect.PETALS_REGISTRY_CLIENT = this.prca.getPetalsRegistryClientFactory().createClient("localhost",
                Constants.DEFAULT_PORT, Constants.DEFAULT_CREDENTIALS_GROUP, Constants.DEFAULT_CREDENTIALS_PASSWORD);
        shell.onConnectionEstablished();
        try {
            final Command command = new DeleteTopology();
            command.setShell(shell);
            command.execute(new String[] { "-z" });
            fail("CommandBadArgumentNumberException is not thrown.");
        } catch (final CommandBadArgumentNumberException e) {
            assertTrue(
                    "Error label is missing.",
                    e.getMessage().startsWith(
                            CommandBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        } finally {
            Connect.PETALS_REGISTRY_CLIENT = null;
        }
    }

    /**
     * Test of the command {@value DeleteTopology#NAME} against argument management, where:
     * <ul>
     * <li>the topology option is missing</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the missing option.
     * </p>
     */
    @Test
    public void errorTopologyOptionMissing() throws CommandException, ConnectionErrorException {

        final Shell shell = new StringStreamShell();

        Connect.PETALS_REGISTRY_CLIENT = this.prca.getPetalsRegistryClientFactory().createClient("localhost",
                Constants.DEFAULT_PORT, Constants.DEFAULT_CREDENTIALS_GROUP, Constants.DEFAULT_CREDENTIALS_PASSWORD);
        shell.onConnectionEstablished();
        try {
            final Command command = new DeleteTopology();
            command.setShell(shell);
            command.execute(new String[] {});
            fail("CommandMissingOptionsException is not thrown.");
        } catch (final CommandMissingOptionsException e) {
            assertCommandMissingOptionsException(e, DeleteTopology.TOPOLOGY_SHORT_OPTION);
        } finally {
            Connect.PETALS_REGISTRY_CLIENT = null;
        }
    }

    /**
     * Test of the command {@value DeleteTopology#NAME} against argument management, where:
     * <ul>
     * <li>the topology value is missing</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the missing value.
     * </p>
     */
    @Test
    public void errorTopologyValueMissing() throws CommandException, ConnectionErrorException {

        final Shell shell = new StringStreamShell();

        Connect.PETALS_REGISTRY_CLIENT = this.prca.getPetalsRegistryClientFactory().createClient("localhost",
                Constants.DEFAULT_PORT, Constants.DEFAULT_CREDENTIALS_GROUP, Constants.DEFAULT_CREDENTIALS_PASSWORD);
        shell.onConnectionEstablished();
        try {
            final Command command = new DeleteTopology();
            command.setShell(shell);
            command.execute(new String[] { "-" + DeleteTopology.TOPOLOGY_SHORT_OPTION });
            fail("CommandMissingOptionsException is not thrown.");
        } catch (final CommandMissingArgumentException e) {
            assertTrue("Error label is missing.",
                    e.getMessage().startsWith(CommandMissingArgumentException.MISSING_ARGUMENT));
            assertEquals("The option '-" + DeleteTopology.TOPOLOGY_SHORT_OPTION + "' has a missing argument.",
                    DeleteTopology.TOPOLOGY_SHORT_OPTION, e.getOption().getOpt());
            assertTrue("Missing argument is missing in error message.",
                    e.getMessage().endsWith(DeleteTopology.TOPOLOGY_SHORT_OPTION));
        } finally {
            Connect.PETALS_REGISTRY_CLIENT = null;
        }
    }

    /**
     * <p>
     * Test of the command {@value DeleteTopology#NAME} to delete a topology of the Petals Registry when no connection
     * is established when running the command.
     * </p>
     * <p>
     * Expected results: an error occurs about the connection not established.
     * </p>
     */
    @Test(expected = ConnectionRequiredException.class)
    public void executeNotConnected() throws CommandException {

        final StringStreamShell shell = new StringStreamShell();
        final Command command = new DeleteTopology();
        command.setShell(shell);

        command.execute(new String[] {});
    }
}
